# -*- coding: utf-8 -*-

#changement de répertoire
import os
os.chdir("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/text_mining/word2vec/tutoriel")

#chargement des posts dans une liste
f = open("toy_posts.txt","r")
liste = f.readlines()
f.close()

#nombre de posts
print(len(liste))

#contenu des posts
print(liste)

#enlever les "\n"
posts = [s.replace("\n","") for s in liste]
print(posts)

#liste des ponctuations
import string
ponctuations = list(string.punctuation)
print(ponctuations)

#retirer les ponctuations
posts = ["".join([w for w in list(msg) if not w in ponctuations]) for msg in posts]
print(posts)

#mettre tout en minuscule
posts = [msg.lower() for msg in posts]
print(posts)

#installation si la liste des stopwords n'est pas dispo
#import nltk
#nltk.download() #cf. CORPORA / STOPWORDS

#charger les stopwords
from nltk.corpus import stopwords
mots_vides = stopwords.words('english')
print(mots_vides)

#transformer les messages en listes par tokenisation
from nltk.tokenize import word_tokenize
posts = [word_tokenize(msg)  for msg in posts]
print(posts)

#retirer les stopwords et reformer les messages
posts = [[w for w in msg if not w in mots_vides] for msg in posts]   
print(posts)

#outil pour stemming - racinisation des termes
from nltk.stem import PorterStemmer
ps = PorterStemmer()

#transformation des messages par racinisation des mots
posts = [[ps.stem(w) for w in msg] for msg in posts]
print(posts)

#reformer les messages à partir des listes
posts = [" ".join(msg) for msg in posts]
print(posts)

#import h2o
import h2o
print(h2o.__version__)

#initialisation
h2o.init()
    
#formation d'un Frame H2O - chaque ligne = un document
#transtyper en caractères
df = h2o.H2OFrame(posts)
df = df.ascharacter()
    
#dimension
df.describe()

#affichage de contrôle
print(df.as_data_frame())

#tokenisation - " " séparateur de mots
tokenized = df.tokenize(" ")
tokenized.describe()

#affichage - on a une seule colonne
#chaque document est séparé par un NaN
print(tokenized.as_data_frame())

#estimateur de word2vec - attention aux paramètres
from h2o.estimators.word2vec import H2OWord2vecEstimator
wv = H2OWord2vecEstimator(vec_size=2,min_word_freq=1,window_size=1,epochs=10000)

#apprentissage
wv.train(training_frame=tokenized)

#description attribuée à chaque terme
terms = wv.to_frame().as_data_frame()
print(terms)

#représentation graphique des termes
import matplotlib.pyplot as plt

fig, axes = plt.subplots(figsize=(10,10))
axes.set_xlim(-3.5,+1.0)
axes.set_ylim(-1.5,+3.0)
for i in range(terms.shape[0]):
    plt.annotate(terms.Word[i],(terms.V1[i],terms.V2[i]),c='blue')
plt.show()

#chercher les voisins les plus proches
wv.find_synonyms("store",count=2)

#coordonnées de store et imag
import numpy
coord1 = numpy.array(terms.loc[terms.Word=='store'].iloc[0,1:],dtype='float')
coord2 = numpy.array(terms.loc[terms.Word=='databas'].iloc[0,1:],dtype='float')

#similarité cosinus
sim = numpy.dot(coord1,coord2)/(numpy.linalg.norm(coord1)*numpy.linalg.norm(coord2))
print(sim)

#description des documents sur la base des termes
description = wv.transform(tokenized,aggregate_method="AVERAGE")
desc_doc = description.as_data_frame()
print(desc_doc)

#vérification pour post n°3
doc = posts[3]
print(doc)

#à la main pour l'instant -- et c'est bien ça ! (1ere coordonnée)
v1 = (1.0/4.0)*(terms.V1[terms.Word=="imag"].values[0] + terms.V1[terms.Word=="databas"].values[0] + terms.V1[terms.Word=="store"].values[0] + terms.V1[terms.Word=="data"].values[0])
print(v1)

#seconde coordonnée
v2 = 0.0
for w in word_tokenize(doc):
    v2 = v2 + terms.V2[terms.Word==w].values[0]
v2 = v2 / 4.0    
print(v2)

#représentation graphique simultanée avec les termes
fig, axes = plt.subplots(figsize=(10,10))
axes.set_xlim(-3.5,+1.0)
axes.set_ylim(-1.5,+3.0)
#les termes
for i in range(terms.shape[0]):
    plt.annotate(terms.Word[i],(terms.V1[i],terms.V2[i]),c='blue')
#les documents
for i in range(desc_doc.shape[0]):
    plt.annotate(str(i),(desc_doc.C1[i],desc_doc.C2[i]),c='red')
plt.show()
